#!/usr/bin/perl

# author: James Edmondson
# purpose: program to run scriptable DDS benchmarks

# usage:

#
# benchmark.pl [-q qosFile] [-n netFile] [-i implementation] [-type testType]
#              [-minSize #] [-maxSize #] [-primer #] [-statsMessages #]
#              [settingsFile]

# ARG LIST
# 1. -i               implementation (NDDS | TAO-DDS - can be mixed case)
# 2. -n               network settings file
# 3. -q               qos settings file
# 4. -minsize         minimum packet size
# 5. -maxsize         maximum packet size
# 6. -type            type of test to run (throughput | latency)
# 7. -primer          number of primer messages to send before stats start
# 8. -statsMessages   number of messages to log stats for 
# n.                  if any othe parameter is passed, it is treated as a
#                     settings file

use Fcntl ':flock';

require "/export/home/tczar/scripts/scripts.lib";

%settings = ();

&readSettingsFromArgs();

# used for debugging if we need to know hash keys and values
#foreach $key (keys %settings)
#{
#  print $key . " = " . $settings{$key} . "\n";
#}


# setup base path for all executables

$path = "~/DDS/performance/throughput/";

#$settings{'results'} = $path;

#$testing = &getTestDirectory();

#print "Test directory is $testing.\n";
#exit(0);

# add implementation specific path information. notice that implementations
# may be upper or lower case.

$impl = lc($settings{'impl'});

if( $impl eq "ndds" )
{
  $path .= "NDDS/";
}
elsif( $impl eq "tao" || $impl eq "tao-dds" || $impl eq "tao_dds"
     || $impl eq "taodds" )
{
  $impl = 'tao';
  $path .= "TAO_DDS/TCP/";
}
else
{
  die "Sorry, but implementation type '$impl' is not supported" .
      " by this script. Try TAO-DDS or NDDS. Use -h for usage info.\n";
}


# make sure that all the important arguments were provided

$settings{'nodes'} ne "" || die "You must include the number of nodes" . 
                                " to test on in args.\n";
$settings{'subscribers'} ne "" || die "You must include the number of" .
                                " subscribers in args.\n";
$settings{'publishers'}  ne "" || die "You must include the number of" .
                                " publishers in args.\n";

# we are officially starting. all parameters have superficially passed.
# assign a test id.

$settings{'id'} = $testid = &getId("/export/home/tczar/DDS/settings/id.gen");
 
# now, make sure the results directories exist, and go ahead and make our
# test id directory for results.

$resultsDir = "/export/home/tczar/DDS/results/$testid";
mkdir("/export/home/tczar/DDS");
mkdir("/export/home/tczar/DDS/results");
mkdir($resultsDir) || print "unable to create $resultsDir\n";

open(FILE,">/export/home/tczar/DDS/results/$testid/impl.dat");
  print FILE uc($impl) . "\n";
close(FILE);

&writeSettings($resultsDir . "/settings.dat");
&updateTestsListingFile();
#&writeEnvironmentForSSH();

&printSummary();

$NUM_NODES = $settings{'nodes'};

# was a node listing provided? if so, we don't need to generate one.

if( $settings{'nodelisting'} )
{
  $settings{'nodelist'} = $settings{'nodelisting'};
}

if( $settings{'nodelist'} ne 'true'  )
{
  # no node listing was provided by user, so generate our own
  # from knowledge of our own cluster nodes (blade30-44)

  for( $i = 0; $i < $NUM_NODES; $i++ )
  {
    $settings{'node' . $i} = "blade" . ($i + 30) . ".isislab.vanderbilt.edu";
  }
}


# setup the repo

if ($impl ne 'tao')
{
  $settings{'repos'} = 0;
}

for( $i = 0, $j = 0; 
     $j < $settings{'repos'}; 
     $i = ($i + 1) % $NUM_NODES , $j++ )
{
  print "Starting Repo (" . $settings{'node' . $i} . ")...\n";

  $command = "'perl " . $path . 'start_repo.pl' .
             " -r ~/DDS/results/$testid/repo$j" .
             ' -n ' . $settings{'net'} . ' -minSize ' . $settings{'minsize'} .
             ' -maxSize ' . $settings{'maxsize'} . ' -type ' .
             $settings{'type'} . ' -primer ' . $settings{'primer'} .
             ' -statsMessages ' . $settings{'statsmessages'};

  if( $settings{"node$i" . '.qos'} ne "" )
  {
    $command .= ' -q ' . $settings{"node$i" . '.qos'}; 
  }
  else
  {
    $command .= ' -q ' . $settings{'qos'};
  }

  $command .= " & '";

# print $command . "\n";
# sleep(5);

  system('ssh -f tczar@' . $settings{"node$i"} . " $command" .
         ' > ' . $resultsDir . '/repo' . $j . '.log'   . ' 2>&1');
}

# wait for a second

sleep(3);

# setup the subscribers

$temp = $settings{'repos'} + $settings{'publishers'};
$lastsub = $temp + $settings{'subscribers'};
for( $i = $temp % $NUM_NODES, $j = 0; 
     $j < $settings{'subscribers'}; 
     $i = ($i + 1) % $NUM_NODES, $j++ )
{
  print "Starting Subscriber (" . $settings{'node' . $i} . ")...\n";

  touch($resultsDir . "/sub$j" . '.exists');

  $command = "'perl " . $path . 'start_sub.pl' .
             " -r ~/DDS/results/$testid/sub$j" .
             ' -n ' . $settings{'net'}  . ' -minSize ' . $settings{'minsize'} .
             ' -maxSize ' . $settings{'maxsize'} . ' -type ' .
             $settings{'type'} . ' -primer ' . $settings{'primer'} .
             ' -statsMessages ' . $settings{'statsmessages'};


  if( $settings{"node$i" . '.qos'} ne "" )
  {
    $command .= ' -q ' . $settings{"node$i" . '.qos'};
  }
  else
  {
    $command .= ' -q ' . $settings{'qos'};
  }

  $command .= " & '";

#  print $command . "\n";

if( $impl eq 'tao' )
{
  sleep(10);
}


  system('ssh -f tczar@' . $settings{"node$i"} . " $command" .
         ' > ' . $resultsDir . '/sub' . $j . '.log'  . ' 2> '
       . $resultsDir . "/sub$j.err");
}

sleep(2);


# setup the publishers
for( $i = $settings{'repos'} % $NUM_NODES,  $j = 0;
     $j < $settings{'publishers'};
     $i = ($i + 1) % $NUM_NODES, $j++ )
{
  print "Starting Publisher (" . $settings{'node' . $i} . ")...\n";

  touch($resultsDir . "/pub$j" . '.exists');

  $command = "'perl " . $path . 'start_pub.pl' .
             " -r ~/DDS/results/$testid/pub$j" .
             ' -n ' . $settings{'net'}  . ' -minSize ' . $settings{'minsize'} .
             ' -maxSize ' . $settings{'maxsize'} . ' -type ' .
             $settings{'type'} . ' -primer ' . $settings{'primer'} .
             ' -statsMessages ' . $settings{'statsmessages'} ;

  if( $settings{"node$i" . '.qos'} ne "" )
  {
    $command .= ' -q ' . $settings{"node$i" . '.qos'};
  }
  else
  {
    $command .= ' -q ' . $settings{'qos'};
  }

  $command .= " & '";

#  print $command . "\n";
#  sleep(5);


  system('ssh -f tczar@' . $settings{"node$i"} . " $command" .
         ' > ' . $resultsDir . '/pub' . $j . '.log' . ' 2> '
       . $resultsDir . "/pub$j.err");


}

sleep(5);

print "Waiting for processes to terminate (" .
      $settings{'publishers'} . ')(' . $settings{'subscribers'} . ") ...\n";

$done = 0;

$numPubs = $settings{'publishers'};
$numSubs = $settings{'subscribers'};

%errors = {};

while( !$done )
{

  $pubsRunning = 0;
  $subsRunning = 0;

  for( $i = 0; $i < $numPubs; $i++ )
  {
    if( -e $resultsDir . "/pub$i.exists" )
    {
      $pubsRunning++;
    }
    if( !$errors{"pub$i"} &&  -s $resultsDir . "/pub$i.err" )
    {
      $errors{"pub$i"} = 1;

      print "ERROR: Publisher has encountered an error (pub$i.err).\n";
    }
  }

  for( $i = 0; $i < $numSubs; $i++ )
  {
    if( -e $resultsDir . "/sub$i.exists" )
    {
      $pubsRunning++;
    }
    if( !$errors{"sub$i"} && -s $resultsDir . "/sub$i.err" )
    {
      $errors{"sub$i"} = 1;

      print "Subscriber may have encountered an error (check sub$i.err).\n";
    }
  }

  if( $pubsRunning == 0 && $subsRunning == 0 )
  {
    print "All publishers and subscribers have completed. Exiting...\n";
    $done = 1;
  }
}

exit 0;
